"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = babelRunner;

var _fs = _interopRequireDefault(require("fs"));

var _os = _interopRequireDefault(require("os"));

var _path = _interopRequireDefault(require("path"));

var _core = require("@babel/core");

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _createJestRunner = require("create-jest-runner");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isWin32 = _os.default.platform() === 'win32';

async function babelRunner({
  testPath,
  config
}) {
  const start = new Date();
  const name = '@tunnckocore/jest-runner-babel';
  const runnerConfig = Object.assign({}, config.haste[name]);
  let result = null;

  try {
    result = (0, _core.transformFileSync)(testPath, runnerConfig.babel);
  } catch (err) {
    return (0, _createJestRunner.fail)({
      start,
      end: new Date(),
      test: {
        path: testPath,
        title: 'Babel',
        errorMessage: err.message
      }
    });
  }

  runnerConfig.outDir = runnerConfig.outDir || runnerConfig.outdir;

  if (typeof runnerConfig.outDir !== 'string') {
    runnerConfig.outDir = 'dist';
  }

  let {
    rootDir
  } = config;

  let relativePath = _path.default.relative(rootDir, testPath);

  runnerConfig.monorepo = runnerConfig.monorepo || relativePath.startsWith('packages');

  if (runnerConfig.monorepo) {
    if (isWin32 && relativePath.indexOf('/') < 0) {
      relativePath = relativePath.replace(/\\/g, '/');
    }

    const segments = relativePath.split('/');

    while (segments.length > 3) segments.pop();

    rootDir = _path.default.join(rootDir, ...segments);

    if (rootDir.endsWith('src')) {
      relativePath = _path.default.relative(rootDir, testPath);
      rootDir = _path.default.dirname(rootDir);
    }
  }

  if (relativePath.startsWith('src')) {
    relativePath = _path.default.relative(_path.default.join(rootDir, 'src'), testPath);
  }

  let outDir = _path.default.resolve(rootDir, runnerConfig.outDir);

  let outFile = _path.default.join(outDir, relativePath);

  outDir = _path.default.dirname(outFile);

  const basename = _path.default.basename(outFile, _path.default.extname(outFile));

  outFile = _path.default.join(outDir, `${basename}.js`);

  _mkdirp.default.sync(_path.default.dirname(outFile));

  _fs.default.writeFileSync(outFile, result.code);

  return (0, _createJestRunner.pass)({
    start,
    end: new Date(),
    test: {
      path: outFile
    }
  });
}